
/***************************************************
* C source with CCS
*  File Name: 20151010_level_guage_fsr_sensor.c
*  Description: check potentiometer and NTC sensor out
*  , and turn on yellow LED and relay
*  PIN A0 na
*  PIN A1 for potentiometer
*  PIN A2 for fsr
*  PIN A3 for start
*  PIN A4 for Relay
*  PIN A5 for LED
*  COPYRIGHT 2015 MYCOMKITS.COM, owned by CNET LIMITED
*  $BEv%W%m%0%i%`$NCx:n8"$O!"@=:n<T!V%^%$%3%s%-%C%H%I%C%H%3%`1?1D!!M-8B2q<R%/%M%C%H!W$K5"B0$7$^$9!#(B
*  $BCx:n8"$rJ|4~$7$F$$$^$;$s$,!"Ev%W%m%0%i%`$r;H$C$?3X=,$NCf$G%W%m%0%i%`$r<+M3$KJQ99$7$F$*;H$$$/$@$5$$!#(B
***************************************************/
// include header file
#include   <16f1825.h>
#DEVICE ADC=10
// settings
#fuses   INTRC_IO, NOWDT, NOMCLR, NOBROWNOUT, PUT, NOLVP, NOPROTECT, NODEBUG
#fuses   NOCPD, NOCLKOUT, NOWRT, NOIESO, NOFCMEN, PLL_SW, NOSTVREN
#use delay(CLOCK = 4000000)
//
#define LED1 PIN_C2
#define LED3 PIN_C1
#define LED5 PIN_C0
#define LED2 PIN_C3
#define LED4 PIN_C4
#define WARNING PIN_A5
#define RELAY PIN_A4
//
long value_setting=512;
long value_sensor=512;
long value_s[17];
int hyste = 0;
int i;
int m;
int j=16;
float log_data;
float rt_temp; //
long weight1=150;
long weight2=275;
long weight3=400;
long weight4=525;
long weight5=650;
// LOG calclation table
const float r_value[23] = {23635.92567, 15161.78685, 10000.0, 6764.289855, 4682.171292,
   3309.912751, 2385.430628, 1749.917183, 1304.847807, 987.7513058, 758.2094321, 589.5776639, 463.9827971, 369.239262,
    269.9123, 241.0803, 197.53, 163.2266, 135.9523, 114.0936, 96.42013, 82.02448, 70.21388};
const float log_value[23] = {10.07052311, 9.626533518, 9.210340372, 8.819412562,
   8.451517233, 8.104677109, 7.777134944, 7.467323742, 7.17384169, 6.895430951, 6.630959643, 6.379406457, 6.139847476,
   5.911444841, 5.693437, 5.48513, 5.285891, 5.095139, 4.912345, 4.73702, 4.568715, 4.407018, 4.251546};
//
// prototyping
#separate
void get_sensor();
#separate
void get_setting();
#separate
void initializing();
#separate
void drive_led();
// main
void main()
{
   initializing();   //ADC port initialize
   output_low(RELAY);   //active high, Relay off
   output_high(WARNING);   //active low, LED off
   // main loop
   while(1)
   {
      if(input(PIN_A3)==1)
      {
         delay_ms(50);   //check it again for chattaling
         if(input(PIN_A3)==1)
         {
            get_setting();
            get_sensor();
            //
            if(j > 0) j--;
            else
            {
                drive_led();
                //
                if(value_sensor > value_setting*0.9)
                {
                   output_low(PIN_A5);   // LED on
                }
                else
                {
                   output_high(PIN_A5);   // LED off
                }
                if(value_sensor > value_setting)
                {

                   if(hyste == 0)
                   {
                      output_high(PIN_A4);   // Relay on
                      hyste = 50;
                   }
                }
                //else
                else if(hyste == 0)
                {
                   output_low(PIN_A4);   // Relay off
                   hyste = 50;
                }
            }
         }
      }
   }
}
//
#separate
void get_setting()
{
   set_adc_channel(1);   //must wait 65u
   delay_us(100);
   value_setting=read_adc();   // 0 to 1024, 0.0V to 5.0V
}
//
#separate
void get_sensor()
{
   // moving average
   int i;
      for (i=15; i>0; i--)
      {
         value_s[i] = value_s[i-1];
      }
   set_adc_channel(2);   //must wait 65u
   delay_us(100);
   value_s[0] = read_adc();   //0 to 1024, 0.3V to 4.6V
   //
   value_sensor = 0;
   for (i=0; i<16; i++)
   {
      value_sensor = value_sensor + value_s[i];
   }
   //
   rt_temp = value_sensor;   // 16 times of actual value, almost twice of real register
   //rt_temp = value_sensor / 16;
   if(rt_temp > 16384.0 ) rt_temp = 16384.0;
   //
   m=0;
   //
   //
   while(rt_temp < r_value[m])
   {
      m++;
   }
   //
   log_data = ((rt_temp-r_value[m])/(r_value[m-1]-r_value[m]))*(log_value[m-1]-log_value[m])+log_value[m];
   if(log_data < 4.2515) log_data = 4.2515;
   if(log_data > 10.0705) log_data = 10.0705;
   //
   log_data = (log_data-4.2515)*175.9; // 1024/5.82 = 175.9
   value_sensor = 1024-log_data;
   //
   //
}
// system intializing
#separate
void initializing()
{
    set_tris_a(0x0F);   //A0 to 3 are input, 4, 5 are output
    set_tris_c(0xE0);   //C0 to 4 are output, 5 is input
    //setup_comparator(NC_NC);
    setup_adc(DAC_OFF);
    // A/D converter initialize AN1 and AN2
    setup_adc_ports(sAN1 | sAN2 | VSS_VDD);   // gnd to 5v
    setup_adc(ADC_CLOCK_DIV_8);         // 2usec
    // timer 1 intialize for hysteresis count down
    setup_timer_1(T1_INTERNAL|T1_DIV_BY_8);
    set_timer1(0xF4F0);              // timer 1 setting almost 5msec
    //
    output_low(LED1);
    output_low(LED2);
    output_low(LED3);
    output_low(LED4);
    output_low(LED5);
    output_high(WARNING);
    output_low(RELAY);
    //int i;
    //for (i=0; i<16; i++)
    //{
    //    value_s[i] = 0xFFFF;
    //}
    enable_interrupts(INT_TIMER1);   // timer 1 enable
    enable_interrupts(GLOBAL);       // global enable
}
// timer 1 inturruption hysteresis count down
#int_timer1
void isr1(void)
{
   set_timer1(0xF4F0);             // timer 1 almost 5msec
   //
   if(hyste > 1) hyste = hyste - 1;
   else hyste = 0;
   //
}
//
#separate
void drive_led()
{
    if(value_sensor > weight1) output_high(LED1);
    else
    {
        output_low(LED1);
        output_low(LED2);
        output_low(LED3);
        output_low(LED4);
        output_low(LED5);
    }
    if(value_sensor > weight2) output_high(LED2);
    else
    {
        output_low(LED2);
        output_low(LED3);
        output_low(LED4);
        output_low(LED5);
    }
    if(value_sensor > weight3) output_high(LED3);
    else
    {
        output_low(LED3);
        output_low(LED4);
        output_low(LED5);
    }
    if(value_sensor > weight4) output_high(LED4);
    else
    {
        output_low(LED4);
        output_low(LED5);
    }
    if(value_sensor > weight5) output_high(LED5);
    else
    {
        output_low(LED5);
    }
}
